@@ -201,7 +201,7 @@ class ORMExecuteTest(_RemoveListeners, _fixtures.FixtureTest):
         def go(context):
             for elem in context.user_defined_options:
                 if isinstance(elem, SetShardOption):
-                    m1.update_execution_options(_sa_shard_id=elem.payload)
+                    m1.do_some_mock_thing(_sa_shard_id=elem.payload)
 
         stmt = select(User).options(
             loader_opt(User.addresses).options(loader_opt(Address.dingaling)),
@@ -217,21 +217,15 @@ class ORMExecuteTest(_RemoveListeners, _fixtures.FixtureTest):
             loader_opt(User.addresses).options(loader_opt(Address.dingaling)),
             SetShardOption("some_other_shard"),
         )
+
         for u in s.execute(stmt).unique().scalars():
             for a in u.addresses:
                 a.dingaling
         eq_(
             m1.mock_calls,
-            (
-                [call.update_execution_options(_sa_shard_id="some_shard")]
-                * num_opts
-            )
+            ([call.do_some_mock_thing(_sa_shard_id="some_shard")] * num_opts)
             + (
-                [
-                    call.update_execution_options(
-                        _sa_shard_id="some_other_shard"
-                    )
-                ]
+                [call.do_some_mock_thing(_sa_shard_id="some_other_shard")]
                 * num_opts
             ),
         )
@@ -2084,6 +2078,35 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         s = fixture_session()
         assert my_listener in s.dispatch.before_flush
 
+    @testing.combinations(True, False, argnames="m1")
+    @testing.combinations(True, False, argnames="m2")
+    @testing.combinations(True, False, argnames="m3")
+    @testing.combinations(True, False, argnames="use_insert")
+    def test_sessionmaker_gen_after_session_listen(
+        self, m1, m2, m3, use_insert
+    ):
+        m1 = Mock() if m1 else None
+        m2 = Mock() if m2 else None
+        m3 = Mock() if m3 else None
+
+        if m1:
+            event.listen(Session, "before_flush", m1, insert=use_insert)
+
+        factory = sessionmaker()
+
+        if m2:
+            event.listen(factory, "before_flush", m2, insert=use_insert)
+
+        if m3:
+            event.listen(factory, "before_flush", m3, insert=use_insert)
+
+        st = factory()
+        st.dispatch.before_flush()
+
+        for m in m1, m2, m3:
+            if m:
+                eq_(m.mock_calls, [call()])
+
     def test_sessionmaker_listen(self):
         """test that listen can be applied to individual
         scoped_session() classes."""
